@@ -4,6 +4,9 @@ import time
 from apscheduler.schedulers.blocking import BlockingScheduler
 from pprint import pprint
 import datetime
+import threading
+import schedule
+
 
 class MicroPoll(MicroWriter):
 
@@ -39,9 +42,9 @@ class MicroPoll(MicroWriter):
     def __init__(self, name, func, interval, write_key="invalid_key", base_url=None, verbose=True, func_args=None, mine= False, **kwargs):
         """  Create a stream by polling every 20 minutes, say
             param: name  str   stream name ending in .json
-            func:        function    returns float  (data feed function)
+            get_iex_realtime_price:        function    returns float  (data feed function)
             interval:    int     minutes between polls
-            func_args    dict    optional dict of arguments to be passed to func
+            func_args    dict    optional dict of arguments to be passed to get_iex_realtime_price
             mine:        bool    Supply mine=True if you want to automatically mine MUIDs to help keep the stream alive
         """
         assert self.is_valid_name(name),'name not valid'
@@ -58,7 +61,7 @@ class MicroPoll(MicroWriter):
         print('Created poller. Example value '+str(self.test_value))
 
     def __repr__(self):
-        self_data= {'func':str(self.func),'interval':self.interval,'mining_time':self.mining_time}
+        self_data= {'get_iex_realtime_price':str(self.func),'interval':self.interval,'mining_time':self.mining_time}
         if self.func_args is not None:
             self_data.update(self.func_args)
         return self_data
@@ -106,7 +109,11 @@ class MicroPoll(MicroWriter):
                 else:
                     print('Did not find MUID this time', flush=True)
 
-    def run(self):
+    def run_with_blocking_scheduler(self):
+        """
+          The old way. However this scheduler creates some problems so we are trying
+          schedule instead, for now.
+        """
         data = {'type': 'scheduler', 'scheduler start time': time.time()}
         data.update(self.__repr__())
         self.logger(data=data)
@@ -119,6 +126,40 @@ class MicroPoll(MicroWriter):
         data.update({'stopping time':time.time()})
         self.logger(data=data)
 
+    def run(self, persist=False, use_thread=False):
+        """
+             Run task regularly
+             This will *not* run missed jobs
+        """
+        import os
+        os.environ['GEVENT_SUPPORT']="1"
+        data = {'type': 'scheduler', 'scheduler start time': time.time()}
+        data.update(self.__repr__())
+        self.logger(data=data)
+
+        def job():
+            self.task()
+
+        def run_threaded(job_func):
+            job_thread = threading.Thread(target=job_func)
+            job_thread.start()
+
+        if use_thread:
+            schedule.every(self.interval).minutes.do(run_threaded, job)
+        else:
+            schedule.every(self.interval).minutes.do(job)
+
+        workin = True
+        while workin or persist:
+            try:
+                schedule.run_pending()
+            except Exception as e:
+                workin = False
+            time.sleep(1)
+
+        data.update({'stopping time': time.time()})
+        self.logger(data=data)
+
 
 class ChangePoll(MicroPoll):
 
@@ -176,7 +217,7 @@ class MultiPoll(MicroPoll):
 
     # -------------------------------------------
     #  You may wish to override this method if the
-    #  data retrieval function func() you provide
+    #  data retrieval function get_iex_realtime_price() you provide
     #  does not return [ float ] or [ str ] that is
     #  convertable to float
     # --------------------------------------------
@@ -184,7 +225,7 @@ class MultiPoll(MicroPoll):
     def determine_next_values(self, source_values):
         """ Should receive raw source data and decides what to send, if anything
 
-            :param source_values   [ float ] or whatever type is returned by self.func. Simplest would be [ float ]
+            :param source_values   [ float ] or whatever type is returned by self.get_iex_realtime_price. Simplest would be [ float ]
             :returns  [ float ]  or None
         """
 
@@ -197,18 +238,28 @@ class MultiPoll(MicroPoll):
     #  Maybe don't override the rest
     # --------------------------------------------
 
-    def __init__(self, names, func, interval, write_key="invalid_key", base_url=None, verbose=True, func_args=None, with_copulas=False, **kwargs):
+    def __init__(self, names, func, interval, write_key="invalid_key", base_url=None, verbose=True, func_args=None, with_copulas=False, secondary_func=None, secondary_func_args=None, **kwargs):
             """  Create a stream by polling every 20 minutes, say
                 param: names  [ str ]    stream name ending in .json
-                func:        function    returns raw data from some live source, ideally [ float ] but you can override determine_next_values method
+                get_iex_realtime_price:        function    returns raw data from some live source, ideally [ float ] but you can override determine_next_values method
                 interval:    int         minutes between polls
-                func_args    dict        optional dict of arguments to be passed to func
+                func_args    dict        optional dict of arguments to be passed to get_iex_realtime_price
 
             """
             assert all( [ self.is_valid_name(name) for name in names]), 'name not valid'
             super().__init__(base_url=base_url or api_url(), write_key=write_key, verbose=verbose, func=func, func_args=func_args, interval=interval, name='never_used.json')
             self.names = names
             self.with_copulas = with_copulas
+            self.secondary_func = secondary_func
+            self.secondary_func_args = secondary_func_args
+
+    def call_secondary_func(self, next_values):
+        if isinstance(self.secondary_func_args,list):
+            return self.secondary_func(next_values, *self.secondary_func_args)
+        elif isinstance(self.secondary_func_args, dict):
+            return self.secondary_func(next_values, **self.secondary_func_args)
+        else:
+            return self.secondary_func(next_values)
 
     def task(self):
         """ Scheduled task that runs every minute """
@@ -222,13 +273,20 @@ class MultiPoll(MicroPoll):
         data.update({'source_values': source_values,
                      'next_values': next_values,
                      'elapsed after polling': time.time() - start_time})
+
+        # Update the values with secondary get_iex_realtime_price
+        if (self.secondary_func is not None) and (next_values is not None):
+            send_values = self.apply_secondary_func(next_values)
+        else:
+            send_values = next_values
+
         # Send the data
         if next_values is None:
             res = {'operation':'touch','exec':[ self.touch(name=name) for name in self.names ]}
         elif self.with_copulas:
-            res = self.cset(names=self.names, values=next_values )
+            res = self.cset(names=self.names, values=send_values )
         else:
-            res = [ self.set(name=name,value=value) for name,value in zip(self.names, next_values) ]
+            res = [ self.set(name=name,value=value) for name,value in zip(self.names, send_values) ]
 
         data.update({'values': next_values, "res": res, 'elapsed after sending': time.time() - start_time})
         self.logger(data=data)
@@ -237,27 +295,43 @@ class MultiPoll(MicroPoll):
 
 
 
+def default_change_criterion(old_values, new_values):
+    [abs(pp - pv) > 1e-6 for pp, pv in zip(old_values, new_values)]
+
+
 class MultiChangePoll(MultiPoll):
 
    COLD = 0
    WARM = 1
 
-   def __init__(self, names, func, interval, write_key="invalid_key", base_url=None, verbose=True, func_args=None, with_copulas=False, **kwargs):
+   def __init__(self, names, func, interval, write_key="invalid_key", base_url=None, verbose=True, func_args=None, with_copulas=False, change_func=None, change_func_args=None, **kwargs):
             """  Create multiple streams by polling every 20 minutes, say
                 param:
                 names        [ str ]    stream name ending in .json
-                func:        function    returns data from some live source, ideally [ float ] but you can override determine_next_values method
+                get_iex_realtime_price:        function    returns data from some live source, ideally [ float ] but you can override determine_next_values method
                 interval:    int         minutes between polls
-                func_args    dict        optional dict of arguments to be passed to func
+                func_args    dict        optional dict of arguments to be passed to get_iex_realtime_price
                 with_copulas bool        Whether to create derived copula streams, or just individual unrelated streams
+                change_func              Function acting directly on a list of value changes
+                change_func_args         Additional argument to change function
             """
             super().__init__(base_url=base_url or api_url(), write_key=write_key, verbose=verbose, func=func, func_args=func_args, interval=interval, names=names, with_copulas=with_copulas, **kwargs)
             self.prev_values = None
             self.feed_state = MultiChangePoll.COLD
             self.current_values = None
+            self.change_func = change_func
+            self.change_func_args = change_func_args
+
+   def call_change_func(self, value_changes:[float])->[float]:
+        if isinstance(self.change_func_args, list):
+            return self.change_func(value_changes, *self.change_func_args)
+        elif isinstance(self.change_func_args, dict):
+            return self.change_func(value_changes, **self.change_func_args)
+        else:
+            return self.change_func(value_changes)
 
    def alert(self, message):
-       print(message, flush=True)
+           print(message, flush=True)
 
    def determine_next_values(self, source_values):
        if self.feed_state == MultiChangePoll.WARM:
@@ -278,7 +352,12 @@ class MultiChangePoll(MultiPoll):
                        {'type': 'feed_status', 'message': "****  Feed unchanged at " + str(datetime.datetime.now())})
                else:
                    self.prev_values = self.current_values
-                   return value_changes
+                   if self.change_func is not None:
+                       altered_changes = self.call_change_func(value_changes = value_changes)
+                   else:
+                       altered_changes = [ c for c in value_changes ]
+                   return altered_changes
+
        elif self.feed_state == MultiChangePoll.COLD:
            # Wait until feed is back up and values start changing
            self.prev_prev = self.prev_values if self.prev_values else None
