@@ -5,7 +5,9 @@ from pprint import pprint
 
 
 # A simple testing crawler that takes a pass_callback and a fail_callback.
-
+# Video tutorials are available at https://www.microprediction.com/python-1 to help you
+# get started. There's a video explanation of FitCrawler, SequentialCrawler and friends
+# at https://www.microprediction.com/fitcrawler
 
 class ReportingCrawler(MicroCrawler):
 
@@ -42,9 +44,9 @@ class ReportingCrawler(MicroCrawler):
             """ Fail fast """
             return [70]
 
-        def sample(self, lagged_values, lagged_times=None, **ignore):
+        def sample(self, lagged_values, lagged_times=None, delay=None, name=None, **ignore):
             if len(lagged_values or []) > 5:
-                return super().sample(lagged_values=lagged_values, lagged_times=lagged_times)
+                return super().sample(lagged_values=lagged_values, lagged_times=lagged_times, delay=delay, name=name)
             else:
                 return sorted(
                     np.random.randn(self.num_predictions))
@@ -82,13 +84,13 @@ class ReportingCrawler(MicroCrawler):
                     getattr(self,method)()
 
                 method = 'get_cdf'
-                self.get_cdf_lagged(name='cop.json',delay=self.DELAYS[0])
+                self.get_cdf_lagged(name='c5_bitcoin.json',delay=self.DELAYS[0])
                 self.get_cdf_lagged(name='three_body_x.json',delay=self.DELAYS[-1])
 
                 method = 'get_lagged_times'
-                self.get_lagged_times('cop.json')
+                self.get_lagged_times('c5_bitcoin.json')
                 method = 'get_lagged_values'
-                self.get_lagged_values('cop.json')
+                self.get_lagged_values('c5_bitcoin.json')
 
             except Exception as e:
                 return {'error':'setup error','method':method,'message':str(e)}
